<?php
/**
 * Ulaform_Action Class
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL). If you did not
 * receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * $Horde: ulaform/lib/Action.php,v 1.6 2005/01/03 14:35:39 jan Exp $
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.6 $
 * @package Ulaform
 */
class Ulaform_Action {

    /**
     * A hash containing any parameters for the current action driver.
     *
     * @var array $_params
     */
    var $_params = array();

    /**
     * Constructor
     *
     * @param optional array $params  Any parameters needed for this action
     *                                driver.
     */
    function Ulaform_Action($params)
    {
        $this->_params = $params;
    }

    /**
     * Returns a list of available action drivers.
     *
     * @return array  An array of available drivers.
     */
    function getDrivers()
    {
        static $drivers = array();
        if (!empty($drivers)) {
            return $drivers;
        }

        $driver_path = dirname(__FILE__) . '/Action/';
        $drivers = array();

        if ($driver_dir = opendir($driver_path)) {
            while (false !== ($file = readdir($driver_dir))) {
                /* Hide dot files and non .php files. */
                if (substr($file, 0, 1) != '.' && substr($file, -4) == '.php') {
                    $driver = substr($file, 0, -4);
                    $driver_info = Ulaform::getActionInfo($driver);
                    $drivers[$driver] = $driver_info['name'];
                }
            }
            closedir($driver_dir);
        }

        return $drivers;
    }

    /**
     * Attempts to return a concrete Action instance based on $driver.
     *
     * @param string $driver          The type of concrete Action subclass to
     *                                return. This is based on the action
     *                                driver ($driver). The code is dynamically
     *                                included.
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters a
     *                                subclass might need.
     *
     * @return object Ulaform_Action  The newly created concrete Action
     *                                instance, or false on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = basename($driver);
        include_once dirname(__FILE__) . '/Action/' . $driver . '.php';
        $class = 'Ulaform_Action_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such action '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Ulaform_Action instance
     * based on $driver. It wil only create a new instance if no Ulaform_Action
     * instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Ulaform_Action::singleton()
     *
     * @param string $driver          The type of concrete Ulaform_Action
     *                                subclass to return. The is based on the
     *                                action driver ($driver). The code is
     *                                dynamically included.
     *
     * @param optional array $params  A hash containing any additional
     *                                configuration or connection parameters a
     *                                subclass might need.
     *
     * @return mixed  The created concrete Ulaform_Action instance, or false on
     *                error.
     */
    function &singleton($driver, $params = array())
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Ulaform_Action::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
