<?php
/**
 * Ulaform_Driver_sql Class
 *
 * $Horde: ulaform/lib/Driver/sql.php,v 1.27 2005/01/03 14:35:40 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.27 $
 * @package Ulaform
 */
class Ulaform_Driver_sql extends Ulaform_Driver {

    var $_db;

    /**
     * Saves the passed form into the db, either inserting a
     * new form if no form_id is available, or updating an
     * existing form if a form_id has been passed.
     *
     * @param array  $params  An array with the form details.
     *
     * @return mixed          True or PEAR Error.
     */
    function saveForm(&$info)
    {
        if (empty($info['form_id'])) {
            $info['form_id'] = $this->_db->nextId('ulaform_forms');
            if (is_a($info['form_id'], 'PEAR_Error')) {
                return $info['form_id'];
            }
            $sql_sprintf = 'INSERT INTO ulaform_forms (form_id, user_uid, form_name, form_action, form_params, form_onsubmit) VALUES(%s, %s, %s, %s, %s, %s)';
        } else {
            $sql_sprintf = 'UPDATE ulaform_forms SET form_id = %s, user_uid = %s, form_name = %s, form_action = %s, form_params = %s, form_onsubmit = %s' . sprintf(' WHERE form_id = %s', (int)$info['form_id']);
        }

        /* Serialize the form params. */
        require_once 'Horde/Serialize.php';
        $info['form_params'] = Horde_Serialize::serialize($info['form_params'], SERIALIZE_UTF7_BASIC);

        $sql = sprintf($sql_sprintf,
                       (int)$info['form_id'],
                       $this->_db->quote(Auth::getAuth()),
                       $this->_db->quote($info['form_name']),
                       $this->_db->quote($info['form_action']),
                       $this->_db->quote($info['form_params']),
                       $this->_db->quote($info['form_onsubmit']));
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::saveForm(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return $info['form_id'];
    }

    /**
     * Saves the passed field into the db, either inserting
     * a new field if no field_id is available, or updating
     * an existing field if a field_id has been passed.
     * If no form_id is available will return with error.
     *
     * @param array  $params  An array with the field details.
     *
     * @return mixed          True or PEAR Error.
     */
    function saveField(&$info)
    {
        if (empty($info['form_id'])) {
            return PEAR::raiseError(_("Missing form"));
        }

        if (empty($info['field_id'])) {
            $info['field_id'] = $this->_db->nextId('ulaform_fields');
            if (is_a($info['field_id'], 'PEAR_Error')) {
                return $info['field_id'];
            }
            if (empty($info['field_order'])) {
                $info['field_order'] = $this->nextFieldOrder($info['form_id']);
            }
            $sql_sprintf = 'INSERT INTO ulaform_fields (field_id, form_id, field_name, field_label, field_type, field_params, field_required, field_readonly, field_desc, field_order) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)';
        } else {
            $sql_sprintf = 'UPDATE ulaform_fields SET field_id = %s, form_id = %s, field_name = %s, field_label = %s, field_type = %s, field_params = %s, field_required = %s, field_readonly = %s, field_desc = %s, field_order = %s' . sprintf(' WHERE field_id = %s', $info['field_id']);
        }

        /* Set up the field data. */
        $info['field_required'] = ($info['field_required'] ? 1 : 0);
        $info['field_readonly'] = ($info['field_readonly'] ? 1 : 0);

        if (!empty($info['field_params'])) {
            require_once 'Horde/Serialize.php';
            $info['field_params'] = Horde_Serialize::serialize($info['field_params']);
        } else {
            $info['field_params'] = null;
        }

        $sql = sprintf($sql_sprintf,
                       $this->_db->quote($info['field_id']),
                       $this->_db->quote($info['form_id']),
                       $this->_db->quote($info['field_name']),
                       $this->_db->quote($info['field_label']),
                       $this->_db->quote($info['field_type']),
                       $this->_db->quote($info['field_params']),
                       $this->_db->quote($info['field_required']),
                       $this->_db->quote($info['field_readonly']),
                       $this->_db->quote($info['field_desc']), 
                       $this->_db->quote($info['field_order']));
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::saveField(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
    }

    /**
     * Gets the next field order position within a form.
     *
     * @param int  $form_id  
     *
     * @return mixed          int or PEAR Error.
     */
    function nextFieldOrder($form_id)
    {
        $sql = sprintf('SELECT MAX(field_order)
                        FROM ulaform_fields
                        WHERE form_id = %s', $form_id);
        $result = $this->_db->getOne($sql);
        $nextorder = $result + 1;
        return $nextorder;
    }

    /**
     * Sets the specified sort order to the fields in a form.
     *
     * @param array  $params  An array with the field ids in
     *                        a specific order.
     *
     * @return mixed          True or PEAR Error.
     */
    function sortFields(&$info)
    {
        if (empty($info['form_id'])) {
            return PEAR::raiseError(_("Missing form"));
        }

        foreach ($info['field_order'] as $field_order => $field_id) {
            $sql = sprintf('UPDATE ulaform_fields
                           SET field_order = %s
                           WHERE field_id = %s',
                                (int)$field_order,
                                (int)$field_id);
            Horde::logMessage('SQL Query by Ulaform_Driver_sql::sortFields(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
            $result = $this->_db->query($sql);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        return true;
    }

    /**
     * Fetches the a list of available forms and the basic data.
     *
     * @return array  An array of the available forms.
     */
    function getForms($form_id = null)
    {
        global $perms;

        $wsql = '';
        if (!is_null($form_id)) {
            $wsql = sprintf(' WHERE form_id = %s', (int)$form_id);
        }

        /* Get the forms. */
        $sql = 'SELECT * FROM ulaform_forms' . $wsql;
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::getForms(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $result = $perms->checkPermissions($result, 'form', PERMS_SHOW, 'form_id');
        return $result;
    }

    /**
     * Fetches the a list of available forms to use.
     *
     * @return array  An array of the available forms.
     */
    function getAvailableForms()
    {
        /* Fetch a list of all forms for now. */
        $sql = 'SELECT * FROM ulaform_forms';
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::getAvailableForms(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        return $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
    }

    /**
     * Fetches all the data specific to the supplied form id.
     *
     * @param integer $form_id  The form id of the form to return.
     *
     * @return array            The form data.
     */
    function getForm($form_id)
    {
        /* Get the main form data. */
        $sql = sprintf('SELECT * FROM ulaform_forms WHERE form_id = %s',
                            (int)$form_id);
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::getForm(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $form = $this->_db->getRow($sql, DB_FETCHMODE_ASSOC);
        if (is_a($form, 'PEAR_Error')) {
            Horde::logMessage($form, __FILE__, __LINE__, PEAR_LOG_ERROR);
            return $form;
        } elseif (empty($form)) {
            return false;
        }

        /* Unserialize the form params. */
        require_once 'Horde/Serialize.php';
        $form['form_params'] = Horde_Serialize::unserialize($form['form_params'], SERIALIZE_UTF7_BASIC);

        return $form;
    }

    /**
     * Fetches the fields for a particular form.
     *
     * @param integer $form_id  The form id of the form to return.
     *
     * @return array            The fields.
     */
    function getFields($form_id, $field_id = null)
    {
        $wsql = '';

        /* Get a specific field only. */
        if (!is_null($field_id)) {
            $wsql = sprintf(' AND field_id = %s', $field_id);
        }

        /* Do the sql. */
        $sql = sprintf('SELECT * FROM ulaform_fields WHERE form_id = %s%s
                       ORDER BY field_order',
                            (int)$form_id,
                            $wsql);
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::getFields(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $results = $this->_db->query($sql);

        /* We may need this. */
        require_once 'Horde/Serialize.php';

        $fields = array();
        while ($field = $results->fetchRow(DB_FETCHMODE_ASSOC)) {
            /* If no internal name set, generate one using field_id. */
            if (empty($field['field_name'])) {
                $field['field_name'] = 'field_' . $field['field_id'];
            }

            /* Check if any params and unserialize, otherwise return null. */
            if (!empty($field['field_params'])) {
                $field['field_params'] = Horde_Serialize::unserialize($field['field_params']);
            } else {
                $field['field_params'] = null;
            }
            $fields[] = $field;
        }

        return $fields;
    }

    /**
     * Deletes a form and all of its fields from the database.
     *
     * @param integer $form_id  The form id of the form to delete.
     *
     * @return boolean          True on success, otherwise PEAR error.
     */
    function deleteForm($form_id)
    {
        /* Delete the form. */
        $sql = sprintf('DELETE FROM ulaform_forms WHERE form_id = %s',
                       (int)$form_id);
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::deleteForm(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $delete = $this->_db->query($sql);
        if (is_a($delete, 'PEAR_Error')) {
            return $delete;
        }

        /* Delete the fields for this form. */
        $sql = sprintf('DELETE FROM ulaform_fields WHERE form_id = %s',
                       (int)$form_id);
        Horde::logMessage('SQL Query by Ulaform_Driver_sql::deleteForm(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $delete = $this->_db->query($sql);
        if (is_a($delete, 'PEAR_Error')) {
            return $delete;
        }

        return true;
    }

    /**
     * Deletes a field from the database.
     *
     * @param integer $field_id  The field id of the field to delete.
     *
     * @return boolean           True on success, otherwise PEAR error.
     */
    function deleteField($field_id)
    {
        /* Delete the field. */
        $sql = sprintf('DELETE FROM ulaform_fields WHERE field_id = %s',
                       (int)$field_id);

        Horde::logMessage('SQL Query by Ulaform_Driver_sql::deleteField(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        return $this->_db->query($sql);
    }

    function initialise()
    {
        global $registry;

        Horde::assertDriverConfig($this->_params, 'sql',
            array('phptype', 'hostspec', 'username', 'database'));

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            Horde::fatal($this->_db, __FILE__, __LINE__);
        }

        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

        return true;
    }

}
